function yeast_fixy0andK
%YEAST_FIXY0ANDK.M
% 
% function mapping out the error as two parameters vary in a 
% generalised logistic growth model
%
% We try to fit parameters for a generalised logistic growth model
% dx/dt = r*x*(1-(x/K)^v)
% which has solution
% x(t) = K/(1+((K/x0)^v-1)*exp(-r*v*t))^(1/v)
%
% Here we fix x0 and K (as these seem to be quite robust values), and 
% we find the error as r and v vary 

% read in the data from Sheet2 of the Excel spreadsheet
[NUMERIC,TXT,RAW]=xlsread('CEN-PK113-5D.xls','Sheet2','','basic');
% the first row of NUMERIC includes a header row, so discard this
yeast = NUMERIC(2:end,:);
% yeast should now have 94 rows and 6 columns (time, 3 replicates, mean and stddev)
size(yeast)

% what is the index for time=1500? 
i1500 = find(yeast(:,1)==1500); 
% beyond this point the data looks dodgy...
yeast = yeast(1:i1500,:);

% open figure 1
figure(1)
% clear the figure
clf
set(gca,'fontsize',16)
% plot the three replicates: optical density as a function of time 
plot(yeast(:,1),yeast(:,2:4),'linewidth',2)
% freeze the plot so we can add additional plots
hold on
% put a grid on the plot
grid on

% assign the time data to t
t=yeast(:,1);
% the fifth column is the mean o.d.
x_exp=yeast(:,5);

% for initial population we use the mean from the data
x0 = mean(yeast(1,2:4));
% we pick K
K = 0.84;

% for yeast_fixy0andK.eps
% set ranges for r and v
rr = 0.0:0.001:0.1;
vv = 0:0.01:1;
% loop through all pairs of r and v values, calculating the squared error
% for each
for i = 1:length(rr)
    for j = 1:length(vv)
        r = rr(i);
        v = vv(j);
        lsee(i,j) = objAb(t,x_exp,r,K,v,x0);
    end
end
min(min(lsee))

figure(3)
clf
% plot an image with colour indicating the size of the error
imagesc(vv,rr,lsee)
colorbar
hold on
% add contours as well
[cs,h] = contour(vv,rr,lsee,[0.05 0.5 5],'w'); 
% label the contours
clabel(cs,h,'fontsize',15,'color','w','rotation',0,'labelspacing',288)
% label the axes
xlabel('v','fontsize',16)
ylabel('r','fontsize',16)
set(gca,'fontsize',16)
set(gca,'ydir','normal')
title('Squared error for yeast data, fixed x0 and K')


    % ---------------------------------------------------------------
    function lse=objAb(t,x_exp,r,K,v,x0)

    %use to create a new curve
    x = genlogsol(t,r,K,v,x0);
    %calculate objective function (squared error)
    lse=sum((x-x_exp).^2);

    end

    % ---------------------------------------------------------------
    function x=genlogsol(t,r,K,v,x0)
        x = K./(1+((K/x0)^v-1)*exp(-r*v*t)).^(1/v);
    end

end
